/*

File: library.c
Copyright (C) 2000-2002 Neil Cafferkey

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA 02111-1307, USA.

*/


#include <exec/types.h>
#include <exec/resident.h>
#include <utility/utility.h>
#include "initializers.h"

#include <proto/exec.h>
#include "library.h"
#include "tuple_protos.h"



#define UTILITY_VERSION 36


/* Private prototypes */
static struct Library *LibInit(struct PCCardBase *lib_base, APTR seg_list, struct ExecBase *base);
static struct Library *LibOpen();
static APTR LibClose(void);
static APTR LibExpunge(void);
static APTR LibReserved();
static VOID DeleteLibrary(struct PCCardBase *base);


#define COMPILE_VERSION		1
#define COMPILE_REVISION	0


/* Return an error immediately if someone tries to run the library */

LONG Main()
{
   return -1;
}



const TEXT library_name[]=PCCARDNAME;
static const TEXT version_string[]=PCCARDNAME " 1.1 (23.11.2002)\n";
static const TEXT utility_name[]=UTILITYNAME;



static const APTR vectors[]=
{
    (APTR)   FUNCARRAY_32BIT_NATIVE, 

   (APTR)LibOpen,
   (APTR)LibClose,
   (APTR)LibExpunge,
   (APTR)LibReserved,
   (APTR)GetTupleInfo,
   (APTR)FreeTupleInfo,
   (APTR)-1
};



struct LibInitStruct
{
	ULONG LibSize;
	void  *FuncTable;
	void  *DataTable;
	void  (*InitFunc)(void);
};




struct LibInitStruct LibInitStruct =
{
   sizeof(struct PCCardBase),
   vectors,
   NULL,
   (void (*)(void)) &LibInit

};



const struct Resident library_rom_tag=
{
   RTC_MATCHWORD,
   (struct Resident *)&library_rom_tag,
   &library_rom_tag + 1,
   RTF_PPC | RTF_AUTOINIT|RTF_EXTENDED,
   COMPILE_VERSION,
   NT_LIBRARY,
   0,
   (STRPTR)library_name,
   (STRPTR)version_string,
   &LibInitStruct
};


#ifdef __MORPHOS__
const ULONG __abox__	= 1;
#endif


LONG NoExecute(void)
{
	return -1;
}


/****i* pccard.library/LibInit ****************************************
*
*   NAME
*	LibInit -- Start up the library.
*
*   SYNOPSIS
*	lib_base = LibInit(lib_base,seg_list)
*
*	struct PCCardBase *LibInit(struct PCCardBase *,APTR);
*
*   FUNCTION
*
*   INPUTS
*
*   RESULT
*
*   EXAMPLE
*
*   NOTES
*
*   BUGS
*
*   SEE ALSO
*
****************************************************************************
*
*/

static struct Library *LibInit(struct PCCardBase *base, APTR seg_list, struct ExecBase *ebase)
{
   BOOL success=TRUE;

   //KPrintF("LibInit start\n");
	
   base->seg_list = seg_list;
   base->sys_base = ebase;
   base->library.lib_Revision = 1;

  
 
   base->utility_base = (APTR)OpenLibrary(utility_name,UTILITY_VERSION);
   if((FALSE)||(base->utility_base==NULL))
      success=FALSE;

   if(!success)
   {
      DeleteLibrary(base);
      base=NULL;
   }

   
   // KPrintF("LibInit end\n");
   return &base->library;
}



/****i* pccard.library/LibOpen *********************************************
*
*   NAME
*	LibOpen -- .
*
*   SYNOPSIS
*	library = LibOpen()
*
*	struct PCCardBase *LibOpen(VOID);
*
*   FUNCTION
*
*   INPUTS
*
*   RESULT
*
*   EXAMPLE
*
*   NOTES
*
*   BUGS
*
*   SEE ALSO
*
****************************************************************************
*
*/

static struct Library *LibOpen()
{
	
   struct PCCardBase *base = (struct PCCardBase *)REG_A6;
   //KPrintF("LibOpen start\n");
   Forbid();

   base->library.lib_OpenCnt++;
   base->library.lib_Flags&=~LIBF_DELEXP;
   Permit();

   //KPrintF("LibOpen end\n");
   return &base->library;
}



/****i* pccard.library/LibClose ********************************************
*
*   NAME
*	LibClose -- .
*
*   SYNOPSIS
*	seg_list = LibClose()
*
*	APTR LibClose(VOID);
*
*   FUNCTION
*
*   INPUTS
*
*   RESULT
*
*   EXAMPLE
*
*   NOTES
*
*   BUGS
*
*   SEE ALSO
*
****************************************************************************
*
*/

static APTR LibClose(void)
{
   struct PCCardBase *base = (struct PCCardBase *)REG_A6;
   APTR seg_list=NULL;

   /* Expunge the library if a delayed expunge is pending */

   if((--base->library.lib_OpenCnt)==0)
   {
      if((base->library.lib_Flags&LIBF_DELEXP)!=0)
         seg_list=LibExpunge();
   }

   return seg_list;
}



/****i* pccard.library/LibExpunge ******************************************
*
*   NAME
*	LibExpunge -- .
*
*   SYNOPSIS
*	seg_list = LibExpunge()
*
*	APTR LibExpunge();
*
*   FUNCTION
*
*   INPUTS
*
*   RESULT
*
*   EXAMPLE
*
*   NOTES
*
*   BUGS
*
*   SEE ALSO
*
****************************************************************************
*
*/

static APTR LibExpunge()
{
   APTR seg_list;
   struct PCCardBase *base = (struct PCCardBase *)REG_A6;

   if(base->library.lib_OpenCnt==0)
   {
      seg_list=base->seg_list;
      Remove((APTR)base);
      DeleteLibrary(base);
   }
   else
   {
      base->library.lib_Flags|=LIBF_DELEXP;
      seg_list=NULL;
   }

   return seg_list;
}



/***** pccard.library/LibReserved ************************************
*
*   NAME
*	LibReserved -- For future expansion.
*
*   SYNOPSIS
*	result = LibReserved()
*
*	APTR LibReserved();
*
*   FUNCTION
*
*   INPUTS
*
*   RESULT
*
*   EXAMPLE
*
*   NOTES
*
*   BUGS
*
*   SEE ALSO
*
****************************************************************************
*
*/

static APTR LibReserved()
{
   return NULL;
}



/****i* pccard.library/DeleteLibrary ***************************************
*
*   NAME
*	DeleteLibrary -- Shuts down the library.
*
*   SYNOPSIS
*	DeleteLibrary()
*
*	VOID DeleteLibrary(VOID);
*
*   FUNCTION
*
*   INPUTS
*
*   RESULT
*
*   EXAMPLE
*
*   NOTES
*
*   BUGS
*
*   SEE ALSO
*
****************************************************************************
*
*/

static VOID DeleteLibrary(struct PCCardBase *base)
{
   UWORD neg_size,pos_size;

   /* Close libraries */

   if(base->utility_base!=NULL)
      CloseLibrary((APTR)base->utility_base);

   /* Free library's memory */

   neg_size=base->library.lib_NegSize;
   pos_size=base->library.lib_PosSize;
   FreeMem((UBYTE *)base-neg_size,pos_size+neg_size);

   return;
}



